# TCL database library along with TK interfaces ver 2.2.4
# Started 2-18-1999 by Ron Murawski
# Not a full database, and probebly never will be

namespace eval KobazDB {
set a KobazDatabase

namespace export dbCreate
namespace export dbClose
namespace export dbField
namespace export dbFields
namespace export dbRecord
namespace export dbBinarySearch
namespace export dbMatchingRecords
namespace export dbNumRecs
namespace export dbList
namespace export dbLoaded
namespace export dbSort
namespace export dbWrite
namespace export dbRead
namespace export dbAppend
namespace export dbModify
namespace export dbDelete

source scripts/kobazscripts/KobazDep.tcl
kbprovide $a
kbrequiretcl 8.1

if {![info exists firstload]} {
    if {![info exists schema(opendbs)]} {
	set schema(opendbs) ""
    }
    set firstload         "1"
}

#  dbScrolledListbox {name args}
#
#  name: Frame which contains the scrolled list box widget
#  args: (Opt) Extra options for the builtin listbox widget
#
#  Return: $name.list (listbox)
#
#  +---------------------+
#  | +-------------+ /|\ |
#  | |             |  |  |
#  | |     List    | V   |
#  | |             | Scr |
#  | |     Box     | oll |
#  | |             |  |  |
#  | +-------------+ \|/ |
#  | <-- HScroll --> Pad |
#  +---------------------+
#
#  Creates: main frame ($name), secondary frame ($name.bottom)
#      main frame: listbox ($name.list), VScroll ($name.sy)
#      secondary frame: HScroll ($name.sx), corner pad ($name.pad)
#
proc dbScrolledListbox { name args } {
    frame $name
    frame $name.bottom

    # Create listbox, horiz/vert scrollbars
    eval { listbox $name.list \
            -yscrollcommand [list $name.sy set] \
            -xscrollcommand [list $name.sx set] } $args
    scrollbar $name.sy -orient vertical   -command [list $name.list yview]
    scrollbar $name.sx -orient horizontal -command [list $name.list xview]
    set pad [expr [$name.sy cget -width] + \
            2 * ([$name.sy cget -bd] + [$name.sy cget -highlightthickness])]
    frame $name.pad -width $pad
    pack $name.bottom -side bottom -fill x
    pack $name.pad -in $name.bottom -side right
    pack $name.sx -in $name.bottom -side bottom -fill x
    pack $name.sy -side right -fill y
    pack $name.list -side left -fill both -expand true

    return $name.list
}

# dbOpen:
#       Takes a full path name and a mode
#       Returns channel if successful else complains and aborts
proc dbOpen { name mode } {
    if {[catch {open $name $mode} channel]} {
        return -code error "$channel"
    }
    return $channel
}

# dbClose:
#       Takes a database alias name and a filename
#       Unloads a database from memory
#       On error, complains and aborts
proc dbClose { alias } {
    variable schema

    set index [lsearch -exact $schema(opendbs) $alias]
    if {$index == "-1"} {return -code error "couldn't close \"$alias\": no such database"}
    set schema(opendbs) [lreplace $schema(opendbs) $index $index]
    
    catch {unset schema($alias,numrecs)}
    catch {unset schema($alias,sortedby)}
    catch {unset schema($alias,numflds)}

    catch {
	for {set x 0} {$x < $schema($alias,numflds)} {incr x} {
	    set fieldname $schema($alias,field$x)
	    catch {unset schema($alias,$fieldname)}
	    catch {unset schema($alias,field$x)}
	}
    }
    catch {unset fieldname}
    catch {unset schema($alias,data)}
    catch {unset schema($alias,filename)}
    catch {unset schema($alias_pri,key)}
    catch {unset schema($alias_rel,$alias_pri)}

    return 0
}

#dbGetSig:
#       Returns database file signature string
proc dbGetSig { } {
    return "Signature: RMM"
}

# dbFieldNum:
#       Given a database alias and a field name
#       Returns the field number (zero-based)
proc dbFieldNum { alias fieldname } {
    variable schema
    
    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    if {![info exists schema($alias,$fieldname)]} {
        return -code error "field name \"$fieldname\" does't exist"
    }

    return $schema($alias,$fieldname)
}

# dbFieldName
#       Given a database alias and a field number (zero-based)
#       Returns the field name
proc dbFieldName { alias fieldnum } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    if {![info exists schema($alias,field$fieldnum)]} {
        return -code error "field number \"$fieldnum\" does't exist"
    }

    return $schema($alias,field$fieldnum)
}

# dbNumRecs
#       Given a database alias
#       Returns the number of records
proc dbNumRecs { alias } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    return $schema($alias,numrecs)
}

# dbNumFields
#       Given a database alias
#       Returns the number of fields
proc dbNumFields { alias } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    return $schema($alias,numflds)
}

# dbList
#       Returns a list of open databases
proc dbList { } {
    variable schema
    return $schema(opendbs)
}

# dbLoaded
#       Given a database alias
#       Returns 1 if database is in memory
#       Returns 0 if database not in memory
proc dbLoaded { alias } {
    if {[lsearch -exact [dbList] $alias] != "-1"} {
        return 1
    } else {
        return 0
    }
}

# dbSortNum
#       Given a database alias
#       Returns the field number the database is sorted on
proc dbSortNum { alias } {
    variable schema

    if {[lsearch -exact [dbList] $alias] == "-1"} {
        return -code error "database \"$alias\" is not loaded"
    }

    return $schema($alias,sortedby)
}

# dbSortName
#       Given a database alias
#       Returns the field name the database is sorted on
proc dbSortName { alias } {
    variable schema

    if {[lsearch -exact [dbList] $alias] == "-1"} {
        return -code error "database \"$alias\" is not loaded"
    }

    return [dbFieldName $alias [dbSortNum $alias]]
}

# dbReadHeader:
#       Internal procedure
#       Takes an alias name and a channel ID
#       Reads the header from disk or, on error, complains and aborts
proc dbReadHeader { alias channel } {
    variable schema

    # Verify file is actually a database file
    if { [gets $channel line] < 0 } {
        return -code error "database \"$alias\" is not a database file"
    }
    if { [string compare $line [dbGetSig]] != 0 } {
        return -code error "database \"$alias\" is not a database file"
    }

    # get number of records
    if { [gets $channel line] < 0 } {
        return -code error "database \"$alias\" is damaged"
    }

    set schema($alias,numrecs) [string trim [lindex [split $line :] 1]]
    # get sorting info
    set line [gets $channel]
    set schema($alias,sortedby) [string trim [lindex [split $line :] 1]]
    # get number of fields
    if { [gets $channel line] < 0 } {
        return -code error "database \"$alias\" is damaged"
    }
    set schema($alias,numflds) [string trim [lindex [split $line :] 1]]
    for { set i 0 } { $i < $schema($alias,numflds) } { incr i } {
        set line [gets $channel]
        set fieldname [string trim [lindex [split $line :] 1]]
        set schema($alias,$fieldname) $i        ;# zero-based
        set schema($alias,field$i) $fieldname   ;# zero-based
    }
    return 0
}

# dbRead:
#       Takes a database alias name and a filename
#       Reads a database into memory
#       On error, complains and aborts
proc dbRead { alias filename } {
    variable schema

    if {[catch {dbOpen $filename r} channel]} {
        return -code error "dbRead: $channel"
    }

    dbReadHeader $alias $channel
    set schema($alias,data) ""
    set schema($alias,filename) $filename
    for { set i 0 } { $i < $schema($alias,numrecs) } { incr i } {
        set temp ""
        for { set j 0 } { $j < $schema($alias,numflds) } { incr j } {
            if { [gets $channel line] == -1 } {
                return -code error "cannot read record $i, field $j"
            }
            lappend temp [string trim $line]
        }
        lappend schema($alias,data) $temp
    }
    close $channel

    if {[lsearch -exact $schema(opendbs) $alias] == "-1" } {
	lappend schema(opendbs) $alias
    }

    return 0
}

# dbWriteHeader:
#       Internal procedure
#       Takes an alias name and a channel ID
#       Writes database header to disk
proc dbWriteHeader { alias channel } {
    variable schema

    puts $channel [dbGetSig]
    puts $channel "Number of Records: $schema($alias,numrecs)"
    puts $channel "Sorted by: $schema($alias,sortedby)"
    puts $channel "Number of Fields: $schema($alias,numflds)"
    for { set i 0 } { $i < $schema($alias,numflds) } { incr i } {
        puts $channel "Field $i: $schema($alias,field$i)"
    }
    return 0
}

# dbWrite:
#       Takes an alias name
#       Returns 0 if database successfully written to disk
#       else returns 1
proc dbWrite { alias {new ""}} {
    variable schema

    if {($new != "new") && (![dbLoaded $alias])} {
        return -code error "database \"$alias\" is not loaded"
    }

#    Huh ?
#    set pos      [string last "." $schema($alias,filename)]
#    set tempname [string range $schema($alias,filename) 0 $pos]

    set tempname "${alias}tmp"

    if {[catch {dbOpen $tempname w} channel]} {
        return -code error "$channel"
    }

    dbWriteHeader $alias $channel
    for {set i 0} {$i < $schema($alias,numrecs)} {incr i} {
        set line [lindex $schema($alias,data) $i]
        for {set j 0} {$j < $schema($alias,numflds)} {incr j} {
            puts $channel [lindex $line $j]
        }
    }

    puts $channel ""
    close $channel
    if {[file exists $schema($alias,filename)]} {
        if {[catch {file delete $schema($alias,filename)} error]} {
            return -code error "error overwriting \"$alias\": $error"
        }
    }
    if {[catch {file rename $tempname $schema($alias,filename)} error]} {
        return -code error "error overwriting \"$alias\": $error"
    }
    return 0
}

# dbSortList:
#       Takes a database name and a field name to sort by
#       Returns the sorted database
proc dbSortList { alias fieldname {order "-increasing"} } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set fieldnum [dbFieldNum $alias $fieldname]

    return [lsort -dictionary $order -index $fieldnum $schema($alias,data)]
}

# dbSort:
#       Takes a database name and a field to sort by
#       Sorts the database by the given field
proc dbSort { alias fieldname {order "-increasing"} } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set schema($alias,data)     [dbSortList $alias $fieldname $order]
    set fieldnum                [dbFieldNum $alias $fieldname]
    set schema($alias,sortedby) $fieldnum
    return 0
}

# dbSetRelation:
#       Sets up a relationship between the primary key and a list of related
#       keys. Each key is a list composed of a database alias and a fieldname
#       in that database. The relation formed will cause all databases to go
#       in lockstep with each other. There cannot be more than one primary
#       field per database. The primary key field must be a uniquely valued
#       field. The primary-key database is sorted for fast access.
#       This procedure should be called before entering any data into either database
#       On error, the procedure displays an error message box and aborts
proc dbSetRelation { primary_key  related_key_list } {
    variable schema

    set alias_pri  [lindex $primary_key 0]
    set field_pri [lindex $primary_key 1]
    set key_num_pri [dbFieldNum $alias_pri $field_pri]
    set schema($alias_pri,key) $key_num_pri
    foreach { alias_rel field_rel } $related_key_list {
        set j [dbFieldNum $alias_rel $field_rel]
        set schema($alias_rel,$alias_pri) $j
#       dbErrWin"Relation is: primary{$alias_pri $i} , related{$alias_rel $j}" nonfatal
    }
    return 0
}

# dbCreate:
#    Creates an empty database named alias with the list of fieldnames given
#    The file is immediately saved to disk as pathname and the file is closed
#    The empty database resides in memory and may be referenced by its alias
#    On error, the procedure displays an error message box and aborts
proc dbCreate { alias pathname fieldlist {type ""} } {
    variable schema

    set schema($alias,filename) $pathname
    set schema($alias,numrecs)  "0"
    set schema($alias,sortedby) ""
    set i 0
    foreach field $fieldlist {
        set schema($alias,field$i) $field
        set schema($alias,$field) $i
        incr i
    }

    set schema($alias,numflds) $i
    set schema($alias,data) {}

    if {![dbLoaded $alias]} {
        lappend schema(opendbs) $alias
    }

    if {$type == "nosave"} {
        return 0
    }
    
    if {[catch {dbWrite $alias new} error]} {
        return -code error "$error"
    }
    if {[catch {dbRead $alias $pathname} error]} {
        return -code error "$error"
    }

    return 0
}

# dbAppend:
#       Takes an alias name and the record data
#       Appends the record to the end of the database
proc dbAppend { alias record } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    lappend schema($alias,data) $record
    incr schema($alias,numrecs)
    return 0
}

# dbInsert:
#       Takes an alias name and the record data
#       Inserts the record into database at position recordnum (zero-based)
#       If database is sorted, insert into sorted position
proc dbInsert { alias record {recordnum ""} } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set sortedby $schema($alias,sortedby)
    
    if {$sortedby != ""} {
        set find      [lindex $record $sortedby]

        set sortedby  [lindex [dbFields $alias] $sortedby]
        set recordnum [dbBinarySearch $alias $sortedby $find returnfail]
    }

    set schema($alias,data) [linsert $schema($alias,data) $recordnum $record]
    incr schema($alias,numrecs)
    return 0
}

# dbCompare:
#       Given two strings
#       Returns -1, 0 or 1 depending if str1 was less-than, equal-to
#       or more-than str2 using a dictionary compare
proc dbCompare { str1 str2 } {
    set str1 [string tolower $str1]
    set str2 [string tolower $str2]
    if { $str1 == $str2 } {
        return 0
    }
    set strlist  [list $str1 $str2]
    set strlist  [lsort -dictionary $strlist]
    set firststr [lindex $strlist 0]
    if { $firststr == $str1 } {
        return -1
    }
    return 1
}

# dbBinarySearch:
#       Takes an alias name, a field name and some searchtext
#       Returns the first matching record number (zero-based)
proc dbBinarySearch { alias fieldname searchtext {retfail ""}} {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set lo 0
    set hi $schema($alias,numrecs)
    set fldnum [dbFieldNum $alias $fieldname]

    while { $lo < $hi } {
        set mid [expr { ($hi + $lo) >> 1 }]
        set rec [lindex $schema($alias,data) $mid]
        set fld [lindex $rec $fldnum]
        set cmp [dbCompare $searchtext $fld]

        if { $cmp > 0 } {
            set lo [expr { $mid + 1 }]
        } elseif { $cmp < 0 } {
            set hi $mid  ;# don't use [expr { $mid - 1 }] here
        } else {
            # found a matching record, is it the first one?
            # loop backwards through records til no match
            while { $cmp == 0 } {
                incr mid -1
                # about to look before first record?
                if { $mid < 0 } {
                    break
                }
                set rec [lindex $schema($alias,data) $mid]
                set fld [lindex $rec $fldnum]
                set cmp [dbCompare $fld $searchtext]
            }
            incr mid
            return $mid
        }
    }

    # Return where the binary search failed
    # Eg: where the match should have been
    if {$retfail == "returnfail"} {
        return $lo
    }

    # no match found
    return -1
}

# dbModify:
#       Takes an alias name, field name, recnum, and new data to insert
#       Changes data in a record
proc dbModify { alias fieldname recnum data} {
    variable  schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set rec      [dbRecord   $alias $recnum]
    set fieldnum [dbFieldNum $alias $fieldname]
    set rec      [lreplace $rec $fieldnum $fieldnum $data]

    # If the database is sorted, and we are changing the field
    # that the database is sorted on, then we must change the data
    # then insert it into a sorted position
    if {$schema($alias,sortedby) == $fieldnum} {
        dbDelete $alias $recnum
        dbInsert $alias $rec
    } else {
        set schema($alias,data) [lreplace $schema($alias,data) $recnum $recnum $rec]
    }

    return 0
}

# dbDelete:
#       Takes an alias name, a fieldname and an index
#       Deletes record <index> from database
proc dbDelete { alias recnum } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set schema($alias,data) [ lreplace $schema($alias,data) $recnum $recnum ]

    if { $schema($alias,numrecs) != 0 } {
	incr schema($alias,numrecs) -1
    }

    return 0
}

# dbFind:
#       Takes an alias name, a fieldname and a searchtext string
#       Returns the record number (zero-based) of the glob-style match
proc dbFind { alias fieldname searchtext } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set fldnum [dbFieldNum $alias $fieldname]
    if { $fldnum == $schema($alias,sortedby) } {
        # database sorted on this field
        return [dbBinarySearch $alias $fieldname $searchtext]
    } else {
        # database not sorted on this field, do linear search
        set recnum 0
        foreach record $schema($alias,data) {
            set curfld [lindex $record $fldnum]
            if { [string match $searchtext $curfld] == 1 } {
                return $recnum ;# searchtext found
            }
            incr recnum
        }
    }
    return -1 ;# searchtext not found in database
}

# dbRecord:
#       Given a database name and a record number (zero-based)
#       Returns the record as a list of fields
proc dbRecord { alias recnum } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    return [lindex $schema($alias,data) $recnum]
}

# dbField:
#       Given a database name, a record number and a field name
#       Returns the field information
proc dbField { alias fieldname recnum } {
    variable  schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set rec [dbRecord $alias $recnum]
    return [lindex $rec [dbFieldNum $alias $fieldname]]
}

# dbFields:
#       Given a database name, 
#       Returns names of fields
proc dbFields { alias } {
    variable  schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    for {set x 0} {$x < $schema($alias,numflds)} {incr x} {
        set fieldname $schema($alias,field$x)
        lappend fields $fieldname
    }
    return $fields
}

# dbVitualRecord
#       Given a record in a main database and a list of requested fields
#       Where each fieldlist item is an alias name and a field name
#       Works in conjuction with dbSetRelation to retrieve related fields
#       Returns a list of the requested fields
proc dbVirtualRecord { maindb mainrecnum fieldlist } {
    variable schema

    if {![dbLoaded $maindb]} {
        return -code error "database \"$maindb\" is not loaded"
    }

    set rec [dbRecord $maindb $mainrecnum]
    foreach { alias fldname } $fieldlist {
        set fldnum [dbFieldNum $alias $fldname]
        # requested a field in the main database?
        if { [string compare $alias $maindb] == 0 } {
            lappend retval [lindex $rec $fldnum]
        # otherwise this is a field in a related database (primary key db)
        } else {
            set searchtext [lindex $rec $schema($maindb,$alias)]
            set prifldnum  $schema($alias,key)
            set prifldname [dbFieldName $alias $prifldnum]
            set prirecnum  [dbFind $alias $prifldname $searchtext]
            lappend retval [dbField $alias $fldname $prirecnum]
        }
    }
    return $retval
}

# dbVitualField:
#       Given a record in a main database and a requested field
#       Where the field is a tuple of: alias name and field name
#       Works in conjuction with dbSetRelation to retrieve related fields
#       Returns the field contents requested
proc dbVirtualField { maindb mainrecnum field } {

    if {![dbLoaded $maindb]} {
        return -code error "database \"$maindb\" is not loaded"
    }

    return [dbVirtualRecord $maindb $mainrecnum $field]
}

# dbMatchingRecords:
#       Takes a database name, a field name and a pattern string
#       Returns a list of record numbers that match
#       The pattern string is searched within the requested field
#       using glob-type matching
proc dbMatchingRecords { alias fieldname pattern } {
    variable schema

    if {![dbLoaded $alias]} {
        return -code error "database \"$alias\" is not loaded"
    }

    set matches ""
    set fieldnum [dbFieldNum $alias $fieldname]
    set pattern  [string tolower $pattern]
    set i 0
    foreach record $schema($alias,data) {
        set field [dbField $alias $fieldname $i]
        set field [string tolower $field]
        if { [string match $pattern $field] == 1 } {
#           dbErrWin "$pattern & $field matched" nonfatal
            lappend matches $i
        }
        incr i
    }
    return $matches
}

# dbTable:
#   Takes a main database name, a list of record numbers and a list of fields
#   Where each field is composed of the tuple: database name, field name
#   Returns the requested records in a table to be used in dbBrowse
proc dbTable { alias recnumlist fieldlist } {

    set i 0
    foreach recnum $recnumlist {
        set virt_rec [dbVirtualRecord $alias $recnum $fieldlist]
        set j 0
        foreach fld $virt_rec {
            set db_table($i,$j) $fld
            incr j
        }
        incr i
    }
    return [array get db_table]
}

# dbBrowse:
#   Takes a database table created by dbTable and a list of display options
#   Displays the table of records in a window
proc dbBrowse { title db_table optionlist } {
    variable db__table__db
    
    array set db__table__db $db_table
    set browse [toplevel .browse]
    wm title $browse $title
    eval table $browse.table $optionlist -variable db__table__db -colstretchmode unset
    pack $browse.table -fill x

    return 0
}

proc dbMainMenu { alias } {
    variable schema

    wm title . Books

    button .add    -text "Add a book" -underline 0 -command AddBook
    button .browse -text "Browse book database" -underline 0 -command BrowseBooks
    button .fix    -text "Fix an entry" -underline 0 -command FixEntry
    button .list   -text "List all books by an author" -underline 0 -command ListBooks
    frame .total
    label .total.text -text "Total number of books is"
    label .total.var -textvar schema($alias,numrecs)
    button .quit  -text "Quit" -underline 0 -command {destroy .}

    variable datatable
    set datatable(0,0) Adams
    set datatable(0,1) "Hi there"
    set datatable(0,2) 1999
    set datatable(1,0) Ansel
    set datatable(1,1) "My Book"
    set datatable(1,2) 1998

    table .table -sparsearray 0 -width 3 -height 2 -variable datatable



#   bind . <a> {.add invoke}
#   bind . <A> {.add invoke}
#   bind . <f> {.fix invoke}
#   bind . <F> {.fix invoke}
#   bind . <l> {.list invoke}
#   bind . <L> {.list invoke}
#   bind . <t> {.total invoke}
#   bind . <T> {.total invoke}
#   bind . <q> {.quit invoke}
#   bind . <Q> {.quit invoke}

    pack .total.text .total.var -side left
    pack .add .browse .fix .list .total .quit -fill x
    pack .table

    return 0
}

#dbSetRelation joined_table {auth AuthorID} {books AuthorID}

# SetView:
#          Takes a viewname and a list of field name tuples
#          Sets up a named view that will display the requested fields

#SetView viewname1 joined_table {auth Author} {books Title}
#SetView viewname2 joined_table {auth Author} {books Title} {books Year}
#

#JOIN,auth,book(auth.AuthorID = books.AuthorId)

#auth
#|AuthorID|Author|DOB|
#|1       |A     |adf
#|2       |B     |asdf

#books
#|AuthorID|Title|Year|Type|
#|2       |sdf  |sdf |sdf
#|1       |sdf  |sf  |df
#|1       |asd  |asdf|sad

#a = |books.AuthorID|Title|Year|Type|auth.AuthorID|Author|DOB|

#VIEW,a(Author,Title,Year)

proc BooksByAuthor { auth_name } {
    append auth_name "*"
    set rec       [dbFind  auth Author $auth_name]
    set auth_name [dbField auth Author   $rec]
    set auth_id   [dbField auth AuthorID $rec]
    set rec2      [dbMatchingRecords books AuthorID $auth_id]
    set i 0
    foreach item $rec2 {
        incr i
    }
    set dbtable [dbTable books $rec2 { books Title books Year books Type}]
    dbBrowse "Books by $auth_name" $dbtable [list -cols 3 -rows $i -colwidth 40]

    return 0
}

proc dbGetListboxChoice { } {
    variable choice
    set choice [.authlb.list curselection]

    return 0
}

proc ChooseAnAuthor { } {
    variable schema
    variable choice
    
    dbSort auth Author
    set authlb [dbScrolledListbox .authlb -height 20 -width 35]
    $authlb insert end "None"
    set fldnum [dbFieldNum auth Author]
    foreach rec $schema(auth,data) {
        set fld [lindex $rec $fldnum]
        $authlb insert end $fld
    }
    bind  $authlb <ButtonRelease-1> dbGetListboxChoice
    pack .authlb
    set choice ""
    button .select -text "Select" -command {dbErrWin "[subst $choice]" nonfatal}
    pack .select
    #   focus $authlb
    dbSort auth AuthorID

    return 0
}

putlog "$a.tcl by Kobaz loaded."
}
